@@ -27,8 +27,6 @@ sub from_arguments_to_choices {
     my $invalid_char = $opt->{invalid_char};
     my $more = 0;
     for my $id ( @ids ) {
-        $opt->{length_view_count} = 0;
-        $opt->{length_video_id}   = 0;
         if ( my $channel_id = _user_id( $opt, $id ) ) {
             my $tmp = _yt_list_info( $opt, 'CL', $channel_id );
             next if ! defined $tmp;
