@@ -102,7 +102,10 @@ sub json_to_hash {
 
 sub _prepare_info {
     my ( $opt, $info, $ex, $video_id ) = @_;
-    $opt->{length_video_id} = length $video_id if length $video_id > $opt->{length_video_id};
+    if ( $info->{$ex}{$video_id}{published} && $info->{$ex}{$video_id}{published} =~ /^(\d\d\d\d-\d\d-\d\d)T(\d\d:\d\d:\d\d)/ ) {
+        $info->{$ex}{$video_id}{upload_date} = $1;
+        $info->{$ex}{$video_id}{upload_datetime} = $1 . 'T' . $2;
+    }
     $info->{$ex}{$video_id}{title}           ||= time();
     $info->{$ex}{$video_id}{duration}        ||= '-:--:--';
     $info->{$ex}{$video_id}{upload_date}     ||= '0000-00-00';
@@ -110,10 +113,6 @@ sub _prepare_info {
     $info->{$ex}{$video_id}{playlist_id} //= '';
     $info->{$ex}{$video_id}{uploader_id} //= $info->{$ex}{$video_id}{uploader} // '';
     $info->{$ex}{$video_id}{uploader}    //= $info->{$ex}{$video_id}{uploader_id};
-    if ( $info->{$ex}{$video_id}{published} && $info->{$ex}{$video_id}{published} =~ /^(\d\d\d\d-\d\d-\d\d)T(\d\d:\d\d:\d\d)/ ) {
-        $info->{$ex}{$video_id}{upload_date} = $1;
-        $info->{$ex}{$video_id}{upload_datetime} = $1 . 'T' . $2;
-    }
     if ( $info->{$ex}{$video_id}{duration} =~ /^[0-9]+\z/ ) {
         $info->{$ex}{$video_id}{duration} = sec_to_time( $info->{$ex}{$video_id}{duration}, 1 );
     }
@@ -128,11 +127,11 @@ sub _prepare_info {
         $info->{$ex}{$video_id}{raters} = insert_sep( $info->{$ex}{$video_id}{raters} );
     }
     if ( $info->{$ex}{$video_id}{view_count} ) {
-        $info->{$ex}{$video_id}{view_count_raw} = $info->{$ex}{$video_id}{view_count} // 0;
+        $info->{$ex}{$video_id}{view_count_raw} = $info->{$ex}{$video_id}{view_count};
         $info->{$ex}{$video_id}{view_count} = insert_sep( $info->{$ex}{$video_id}{view_count} );
-        if ( length $info->{$ex}{$video_id}{view_count} > $opt->{length_view_count} ) {
-            $opt->{length_view_count} = length $info->{$ex}{$video_id}{view_count};
-        }
+    }
+    else {
+        $info->{$ex}{$video_id}{view_count} = 0;
     }
 }
 
