@@ -11,18 +11,17 @@ our @EXPORT_OK = qw( download_youtube );
 use Fcntl          qw( LOCK_EX SEEK_END );
 use File::Basename qw( basename );
 use Time::HiRes    qw( gettimeofday tv_interval );
-use Time::Local    qw( timelocal );
 
-use Encode::Locale   qw();
-use File::Touch      qw();
-use LWP::UserAgent   qw();
-use Term::ANSIScreen qw( :cursor :screen );
-use Try::Tiny        qw( try catch );
+use Encode::Locale     qw();
+use LWP::UserAgent     qw();
+use Term::ANSIScreen   qw( :cursor :screen );
+use Term::Choose::Util qw( insert_sep );
+use Try::Tiny          qw( try catch );
 
 use if $^O eq 'MSWin32', 'Win32::Console::ANSI';
 
-use App::YTDL::Data   qw( get_new_video_url );
-use App::YTDL::Helper qw( sec_to_time insert_sep encode_fs encode_stdout HIDE_CURSOR SHOW_CURSOR );
+use App::YTDL::GetData qw( get_new_video_url );
+use App::YTDL::Helper  qw( sec_to_time encode_fs encode_stdout timestamp_to_upload_date HIDE_CURSOR SHOW_CURSOR );
 
 END { print SHOW_CURSOR }
 
@@ -30,30 +29,21 @@ END { print SHOW_CURSOR }
 
 sub download_youtube {
     my ( $opt, $info ) = @_;
-    my $ua = LWP::UserAgent->new(
-        agent         => $opt->{useragent},
-        timeout       => $opt->{timeout},
-        show_progress => 0,
-    );
     my $nr = 1;
-    for my $video_id ( sort { $info->{$a}{count} <=> $info->{$b}{count} } keys %$info ) {
-        for my $fmt ( sort keys %{$info->{$video_id}{video_url}} ) {
-            try {
-                my $file_name_OS = encode_fs( $info->{$video_id}{file_name}{$fmt} );
-                unlink $file_name_OS or die $! if -f $file_name_OS && $opt->{overwrite};
-                _download_video( $opt, $info, $ua, $video_id, $fmt, $nr );
-                if ( $opt->{upload_date_is_file_date} ) {
-                    if ( $info->{$video_id}{upload_datetime} =~ /^(\d\d\d\d)-(\d\d)-(\d\d)T(\d\d):(\d\d):(\d\d)\z/ ) {
-                        my $time = timelocal( $6, $5, $4, $3, $2 - 1, $1 );
-                        my $ref = File::Touch->new( time => $time );
-                        my $count = $ref->touch( $file_name_OS );
-                    }
+    for my $ex ( sort keys %$info ) {
+        for my $video_id ( sort { $info->{$ex}{$a}{count} <=> $info->{$ex}{$b}{count} } keys %{$info->{$ex}} ) {
+            for my $fmt ( sort keys %{$info->{$ex}{$video_id}{file_name}} ) {
+                my $file_name = $info->{$ex}{$video_id}{file_name}{$fmt};
+                try {
+                    unlink encode_fs( $file_name ) or die $!                      if -f encode_fs( $file_name ) && $opt->{overwrite};
+                    _download_video( $opt, $info, $ex, $video_id, $fmt, $nr );
+                    timestamp_to_upload_date( $info, $ex, $video_id, $file_name ) if $opt->{modify_timestamp};
                 }
+                catch {
+                    say "$video_id - $_";
+                };
+                $nr++;
             }
-            catch {
-                say "$video_id - $_";
-            };
-            $nr++;
         }
     }
     return;
@@ -61,17 +51,21 @@ sub download_youtube {
 
 
 sub _download_video {
-    my ( $opt, $info, $ua, $video_id, $fmt, $nr ) = @_;
-    my $video_url     = $info->{$video_id}{video_url}{$fmt};
-    my $file_name     = $info->{$video_id}{file_name}{$fmt};
-    my $file_name_OS  = encode_fs( $file_name );
+    my ( $opt, $info, $ex, $video_id, $fmt, $nr ) = @_;
+    my $video_url     = $info->{$ex}{$video_id}{video_url}{$fmt};
+    my $file_name     = $info->{$ex}{$video_id}{file_name}{$fmt};
     my $file_basename = basename $file_name;
+    my $ua = LWP::UserAgent->new(
+        agent         => $opt->{useragent},
+        timeout       => $opt->{timeout},
+        show_progress => 0,
+    );
     print HIDE_CURSOR;
-    say '  -----' if $nr > 1;
+    my $video_count = sprintf "%*s from %s", length $opt->{nr_videos}, $nr, $opt->{nr_videos};
+    printf "  %s\n", '-' x length $video_count if $nr > 1;
     binmode STDOUT, ':pop';
-    printf "  %s (%s)\n", encode_stdout( $file_basename ), $info->{$video_id}{duration} // '?';
+    printf "  %s (%s)\n", encode_stdout( $file_basename ), $info->{$ex}{$video_id}{duration} // '?';
     binmode STDOUT, ':encoding(console_out)';
-    my $video_count = sprintf "%*s from %s", length $opt->{total_nr_videos}, $nr, $opt->{total_nr_videos};
     local $SIG{INT} = sub {
         print cldown, "\n";
         print SHOW_CURSOR;
@@ -79,8 +73,9 @@ sub _download_video {
     };
     my $p = {};
     my $try = 1;
+
     RETRY: while ( 1 ) {
-        $p->{size}      = -s $file_name_OS // 0;
+        $p->{size}      = -s encode_fs( $file_name ) // 0;
         $p->{starttime} = gettimeofday;
         my $retries = sprintf "%*s/%s", length $opt->{retries}, $try, $opt->{retries};
         if ( $try > 1 ) {
@@ -92,9 +87,9 @@ sub _download_video {
         my $res;
         if ( ! $p->{size} ) {
             my $at = '';
-            open my $fh, '>:raw', $file_name_OS or die $!;
             printf _p_fmt( $opt, "start" ), $video_count, $retries, $at;
-            _log_info( $opt, $info, $video_id, $fmt ) if $opt->{log_info};
+            open my $fh, '>:raw', encode_fs( $file_name ) or die $!;
+            _log_info( $opt, $info, $ex, $video_id, $fmt ) if $opt->{log_info};
             $res = $ua->get(
                 $video_url,
                 ':content_cb' => _return_callback( $opt, $fh, $p ),
@@ -103,8 +98,8 @@ sub _download_video {
         }
         elsif ( $p->{size} ) {
             my $at = sprintf "at %.2fM", $p->{size} / 1024 ** 2;
-            open my $fh, '>>:raw', $file_name_OS or die $!;
             printf _p_fmt( $opt, "start" ), $video_count, $retries, $at;
+            open my $fh, '>>:raw', encode_fs( $file_name ) or die $!;
             $res = $ua->get(
                 $video_url,
                 'Range'       => "bytes=$p->{size}-",
@@ -112,67 +107,9 @@ sub _download_video {
             );
             close $fh or die $!;
         }
-        print up;
-        print cldown;
-        my $status = $res->code;
-        my $dl_time = sec_to_time( int( tv_interval( [ $p->{starttime} ] ) ), 1 );
-        my $size = '';
-        my $avg_speed = '';
-        my $at = '';
-        my $pr_status = 'status ' . $status;
-        my $incomplete = '';
-        if ( $status =~ /^(200|206|416)/ ) {
-            my $file_size = -s $file_name_OS // -1;
-            $avg_speed = sprintf "avg %2sk/s", $p->{kbs_avg} || '--';
-            if ( $p->{total} ) {
-                my $len_total_mb = length( int( $p->{total} / 1024 ** 2 ) ) + 2;
-                $size = sprintf "%*.2fM",  $len_total_mb, $file_size / 1024 ** 2;
-                $at   = sprintf "@%*.2fM", $len_total_mb, $p->{size} / 1024 ** 2;
-                if ( $file_size != $p->{total} ) {
-                    my $should = sprintf "%.2fM", $p->{total} / 1024 ** 2;
-                    my $is     = sprintf "%.2fM", $file_size  / 1024 ** 2;
-                    if ( $should eq $is ) {
-                        $should = insert_sep( $p->{total} );
-                        $is     = insert_sep( $file_size );
-                    }
-                    $incomplete = sprintf " Incomplete: %*s/%s ", length( $should ), $is, $should;
-                    $retries    = sprintf "%*s/%s", length $opt->{retries}, $try, $opt->{retries};
-                }
-            }
-            if ( $status == 416 ) {
-                $dl_time   = ' ' x length '0:00:00';
-                $size      = ' ' x length $size;
-                $avg_speed = ' ' x length $avg_speed;
-                printf _p_fmt( $opt, "status_416" ), $video_count, $retries, $dl_time, $at, $pr_status;
-                last RETRY;
-            }
-            else {
-                if ( $status == 200 ) {
-                    $at        = ' ' x length $at;
-                    $pr_status = ' ' x length $pr_status;
-                }
-                printf _p_fmt( $opt, "status" ), $video_count, $retries, $dl_time, $size, $avg_speed, $at, $pr_status, $incomplete;
-                last RETRY if $p->{total} && $p->{total} == $file_size;
-            }
-        }
-        else {
-            $retries    = sprintf "%*s/%s", length $opt->{retries}, $try, $opt->{retries};
-            $size       = ' ';
-            $avg_speed  = ' ';
-            $at         = ' ';
-            $pr_status  = 'status ' . $status . '  Fetching new video url ...';
-            $incomplete = ' ';
-            my $new_video_url = get_new_video_url( $opt, $info, $video_id, $fmt );
-            if ( ! $new_video_url ) {
-                $pr_status .=  ' failed!';
-            }
-            elsif ( $new_video_url eq $video_url ) {
-                $pr_status .= $res->status_line;
-            }
-            else {
-                $video_url = $new_video_url;
-            }
-            printf _p_fmt( $opt, "status" ), $video_count, $retries, $dl_time, $size, $avg_speed, $at, $pr_status, $incomplete;
+        my $ok = _status_line( $opt, $info, $ex, $video_id, $fmt, $p, $res, $video_count, $try, $retries );
+        if ( $ok ) {
+            last RETRY;
         }
         $try++;
         if ( $try > $opt->{retries} ) {
@@ -186,21 +123,70 @@ sub _download_video {
 }
 
 
-sub _log_info {
-    my ( $opt, $info, $video_id, $fmt ) = @_;
-    my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime;
-    my $id = $info->{$video_id}{extractor_key} eq 'Youtube' ? $video_id : $info->{$video_id}{extractor_key} . ' ' . $video_id;
-    my $log_str = sprintf( "%04d-%02d-%02d %02d:%02d", $year + 1900, $mon + 1, $mday, $hour, $min )
-        . '>  ' . sprintf( "%11s", $id )
-        . ' | ' . ( $info->{$video_id}{uploader_id} // 'xxxxx' )
-        . ' | ' . ( $info->{$video_id}{playlist_id} // '-----' )
-        . ' | ' . ( $info->{$video_id}{upload_date} )
-        . '   ' . basename( $info->{$video_id}{file_name}{$fmt} );
-    open my $log, '>>:encoding(UTF-8)', encode_fs( $opt->{log_file} ) or die $!;
-    flock $log, LOCK_EX     or die $!;
-    seek  $log, 0, SEEK_END or die $!;
-    say   $log $log_str;
-    close $log or die $!;
+sub _status_line {
+    my ( $opt, $info, $ex, $video_id, $fmt, $p, $res, $video_count, $try, $retries ) = @_;
+    my $file_name  = $info->{$ex}{$video_id}{file_name}{$fmt};
+    my $dl_time    = sec_to_time( int( tv_interval( [ $p->{starttime} ] ) ), 1 );
+    my $status     = $res->code;
+    my $pr_status  = 'status ' . $status;
+    my ( $size, $avg_speed, $at, $incomplete ) = ( '' ) x 4;
+    print up;
+    print cldown;
+    if ( $status =~ /^(200|206|416)/ ) {
+        my $file_size = -s encode_fs( $file_name ) // -1;
+        $avg_speed = sprintf "avg %2sk/s", $p->{kbs_avg} || '--';
+        if ( $p->{total} ) {
+            my $len_total_mb = length( int( $p->{total} / 1024 ** 2 ) ) + 2;
+            $size = sprintf "%*.2fM",  $len_total_mb + 3, $file_size / 1024 ** 2;
+            $at   = sprintf "@%*.2fM", $len_total_mb + 3, $p->{size} / 1024 ** 2;
+            if ( $file_size != $p->{total} ) {
+                my $should_size = sprintf "%.2fM", $p->{total} / 1024 ** 2;
+                my $is_size     = sprintf "%.2fM", $file_size  / 1024 ** 2;
+                if ( $should_size eq $is_size ) {
+                    $should_size = insert_sep( $p->{total} );
+                    $is_size     = insert_sep( $file_size );
+                }
+                $incomplete = sprintf " Incomplete: %*s/%s ", length( $should_size ), $is_size, $should_size;
+                $retries    = sprintf "%*s/%s", length $opt->{retries}, $try, $opt->{retries};
+            }
+        }
+        if ( $status == 416 ) {
+            $dl_time   = ' ' x length '0:00:00';
+            $size      = ' ' x length $size;
+            $avg_speed = ' ' x length $avg_speed;
+            printf _p_fmt( $opt, "status_416" ), $video_count, $retries, $dl_time, $at, $pr_status;
+            return 1;
+        }
+        else {
+            if ( $status == 200 ) {
+                $at        = ' ' x length $at;
+                $pr_status = ' ' x length $pr_status;
+            }
+            printf _p_fmt( $opt, "status" ), $video_count, $retries, $dl_time, $size, $avg_speed, $at, $pr_status, $incomplete;
+            return 1 if $p->{total} && $p->{total} == $file_size;
+        }
+    }
+    else {
+        $retries    = sprintf "%*s/%s", length $opt->{retries}, $try, $opt->{retries};
+        $size       = ' ';
+        $avg_speed  = ' ';
+        $at         = ' ';
+        $pr_status  = 'status ' . $status . '  Fetching new video url ...';
+        $incomplete = ' ';
+        my $video_url     = $info->{$ex}{$video_id}{video_url}{$fmt};
+        my $new_video_url = get_new_video_url( $opt, $info, $ex, $video_id, $fmt );
+        if ( ! $new_video_url ) {
+            $pr_status .=  ' failed!';
+        }
+        elsif ( $new_video_url eq $video_url ) {
+            $pr_status .= $res->status_line;
+        }
+        else {
+            $video_url = $new_video_url;
+        }
+        printf _p_fmt( $opt, "status" ), $video_count, $retries, $dl_time, $size, $avg_speed, $at, $pr_status, $incomplete;
+    }
+    return;
 }
 
 
@@ -273,6 +259,26 @@ sub _return_callback {
 }
 
 
+sub _log_info {
+    my ( $opt, $info, $ex, $video_id, $fmt ) = @_;
+    my ( $sec, $min, $hour, $mday, $mon, $year ) = localtime;
+    my $uploader_id = $info->{$ex}{$video_id}{uploader_id};
+    my $playlist_id = $info->{$ex}{$video_id}{playlist_id};
+    my $id = $ex eq 'youtube' ? $video_id : $ex . ' ' . $video_id;
+    my $log_str = sprintf( "%04d-%02d-%02d %02d:%02d", $year + 1900, $mon + 1, $mday, $hour, $min )
+        . '>  ' . sprintf( "%11s", $id )
+        . ' | ' . ( length $uploader_id ? $uploader_id : 'xxxxx' )
+        . ' | ' . ( length $playlist_id ? $playlist_id : '-----' )
+        . ' | ' . ( $info->{$ex}{$video_id}{upload_date} )
+        . '   ' . basename( $info->{$ex}{$video_id}{file_name}{$fmt} );
+    open my $log, '>>:encoding(UTF-8)', encode_fs( $opt->{log_file} ) or die $!;
+    flock $log, LOCK_EX     or die $!;
+    seek  $log, 0, SEEK_END or die $!;
+    say   $log $log_str;
+    close $log or die $!;
+}
+
+
 
 
 1;
