@@ -42,8 +42,11 @@ sub _prepare_print_info {
         }
     }
     my @keys = ( qw( title video_id ) );
-    push @keys, 'extractor' if $ex ne 'youtube';
-    push @keys, qw( author duration raters avg_rating view_count published description keywords ); # categories
+    push @keys, 'extractor'  if $ex ne 'youtube';
+    push @keys, qw( author duration raters avg_rating );
+    push @keys, 'view_count' if $info->{$ex}{$video_id}{view_count};
+    push @keys, 'published'  if $info->{$ex}{$video_id}{upload_date} ne '0000-00-00';
+    push @keys, qw( description keywords ); # categories
     for my $key ( @keys ) {
         next if ! $info->{$ex}{$video_id}{$key};
         $info->{$ex}{$video_id}{$key} =~ s/\R/ /g;
